#
# VLDB submission 'Dependable Cardinality Forecasts for XQuery'
# (Jens Teubner, Torsten Grust, Sebastian Maneth, and Sherif Sakr)
#
# (c) 2008 Jens Teubner, IBM Research, teubner@us.ibm.com
#
require 'algebra'

class ElemConstr < AlgebraOperator

    #
    # Invoke with a hash containing
    #
    # +tagnames+::  left input relation (with tag names)
    # +content+::   right input relation (content expression)
    #
    # Both input relations need to have iter,pos,item schema!
    #
    def initialize(params)

        raise "No left input relation given" unless params[:tagnames]
        raise "No right input relation given" unless params[:content]

        @tagnames = params[:tagnames]
        @content = params[:content]

        raise "Wrong number of parameters in argument #{ @tagnames.nodename }" unless @tagnames.cols.size == 2
        raise "Wrong number of parameters in argument #{ @content.nodename }" unless @content.cols.size == 3

        # set schema
        super(params)

    end

    def infer_cols

        self.cols = [:iter,:pos,:item]

    end

    def infer_const

        self.const = Hash.new
        self.const[:pos] = 1

    end

    def infer_dom
        
        self.dom = Hash.new

        self.dom[:iter] = @tagnames.dom[:iter]
        self.dom[:pos]  = Domain.new :size => 1
        self.dom[:item] = Domain.new :size => self.card

    end

    def infer_path
        
        self.path = Hash.new

    end

    def infer_hist
        
        self.hist = Hash.new

    end

    def infer_card

        self.card = @tagnames.card

    end

    def to_dot_worker

        ret = <<EOT
        #{ nodename } [shape=record,label="ElemConstr (#{ nodename })"];
EOT

        ret
    end

    def children
        [ @tagnames, @content ]
    end

    #
    # Note: Does not (yet?) consider order in its input (i.e., the pos column)
    #
    def evaluate_worker

        res = Result.new self.cols

        tagnames = @tagnames.evaluate
        content = @content.evaluate

        titer = tagnames.col(:iter)
        titem = tagnames.col(:item)

        citer = content.col(:iter)
        cpos  = content.col(:pos)
        citem = content.col(:item)

        tagnames.tuples.each do |tagname|

            new_root = XMLStore::new_elem(tagname[titem])

            # content tuples with matching `iter' value
            # (should sort by `pos', too)
            matching_content = content.tuples.find_all { |t|
                t[citer] == tagname[titer] }

            matching_content.each do |content_tuple|

                node = content_tuple[citem]

                if node.instance_of? Node or node.instance_of? Attribute
                    content_tuple[citem].copy new_root
                else
                    raise "Atomic content not yet supported."
                end

            end

            new_tuple = [ tagname[titer], 1, new_root ]

            res << new_tuple

        end

        res

    end

end
