# #
#  VLDB submission 'Dependable Cardinality Forecasts for XQuery'
#  (Jens Teubner, Torsten Grust, Sebastian Maneth, and Sherif Sakr)
#
#  (c) 2008 Jens Teubner, IBM Research, teubner@us.ibm.com
#
# Representation of (tabular) result
class Result

    attr_reader :cols
    attr_reader :tuples

    def initialize(cols)
        @cols = cols
        @tuples = []
    end

    def <<(tuple)
        if tuple.size != @cols.size
            raise "Tuple has wrong number of elements"
        end
        @tuples << tuple
    end

    def col(colname)
        @cols.index colname
    end

    def to_s

        ret = ""

        colwidths = []

        @cols.each_with_index do |colname,i|

            if @tuples.nil?
                width = 0
            else
                width = (@tuples.map { |tuple| tuple[i].to_s.size }).max
                width = 0 if width.nil?
            end

            if colname.to_s.size > width
                width = colname.to_s.size
            end

            width = 20 if width > 20

            colwidths << width

            ret << "|" unless i == 0

            ret << colname.to_s[0..width-1].center(width + 2)

        end
        ret << "\n"

        colwidths.each_with_index do |width,i|
            ret << "+" unless i == 0
            (width+2).times { ret << "-" }
        end
        ret << "\n"


        @tuples.each do |tuple|
            ret << " "
            tuple.each_with_index do |data,i|
                ret << " | " unless i == 0
                ret << data.to_s.tr("\n\t", ' ')[0..colwidths[i]-1].rjust(colwidths[i])
            end
            ret << "\n"
        end

        ret
    end

    # serialize to XML, tuple by tuple.  Assumes that there's an `item'
    # column (which is the one that's serialized).
    def serialize

        itemcol = col(:item)

        @tuples.each do |tuple|

            item = tuple[itemcol]

            if item.instance_of? Node
                puts item.serialize
            elsif item.instance_of? Attribute
                puts "ATTR: #{ item.serialize }"
            else
                puts item.to_s
            end

        end

    end

end
