##
# VLDB submission 'Dependable Cardinality Forecasts for XQuery'
# (Jens Teubner, Torsten Grust, Sebastian Maneth, and Sherif Sakr)
#
# (c) 2008 Jens Teubner, IBM Research, teubner@us.ibm.com
#
#
# Compute probability as described in Appendix A in the paper
#
# Required input:
#
# +sel+::     Selectivity (parameter s in Appendix A)
# +relsize+:: Size of input relation (parameter R in Appendix A)
# +domsize+:: Input domain size (parameter \gamma_{in} in Appendix A)
#
def probability(params)

    raise "Missing parameter `sel'" unless params[:sel]
    raise "Missing parameter `relsize'" unless params[:relsize]
    raise "Missing parameter `domsize'" unless params[:domsize]

    (1 - ((1 - params[:sel])**(params[:relsize]/params[:domsize])))

end

#
# Evaluate the query specified by the string +query+ and return the
# result as a string.  Uses the Saxon query processor underneath.
#
def eval_xquery(query)

    File.open "#{ $$ }query.xq", "w" do |f|

        f.puts <<EOT
declare namespace saxon = 'http://saxon.sf.net/';
declare option saxon:output "omit-xml-declaration=yes";
EOT
        f.puts query
    end

    result = `java -cp /opt/saxon:/opt/saxon/saxon8sa.jar net.sf.saxon.Query -sa #{ $$ }query.xq`
    raise "Error executing Saxon: #{ $? }" unless $?.success?

    File.delete "#{ $$ }query.xq"

#    puts "Query\n#{ query }\nevaluated to: " + result.split.inspect

    result

end

def is_numeric(path)

    query = <<EOT
    every $x in data( #{ path } )
      satisfies $x castable as xs:double
EOT

    result = eval_xquery(query)

    if result == "true"
        return true
    elsif result == "false"
        return false
    else
        raise "Problem in is_numeric(#{ path }) - got `#{ result }'"
    end

end

def fun_1to1(params)

    implementations = { "fn:number" => FnNumber,
                        "fn:contains" => FnContains,
                        "fn:local-name" => FnLocalName,
                        "fn:ends-with" => FnEndsWith,
                        "multiply" => Multiply,
                        "add" => Add,
                        "fn:substring3" => FnSubstring3
    }

    raise "No input relation given" unless params[:input]
    raise "No output column given" unless params[:res]
    raise "No function name given" unless params[:fun_name]

    #@input = params[:input]
    #@res   = params[:res]

    impl = implementations[params[:fun_name]]

    raise "No implementation for #{ params[:fun_name] }" if impl.nil?

    return impl.new(params)

end
