#
# VLDB submission 'Dependable Cardinality Forecasts for XQuery'
# (Jens Teubner, Torsten Grust, Sebastian Maneth, and Sherif Sakr)
#
# (c) 2008 Jens Teubner, IBM Research, teubner@us.ibm.com
#
require 'algebra'

class Avg < AlgebraOperator

    #
    # Invoke with a hash containing
    #
    # +input+:: the input relation
    #
    def initialize(params)

        raise "No input relation given" unless params[:input]
        raise "No output column specified" unless params[:res]
        raise "No input column specified" unless params[:arg]

        @input  = params[:input]
        @res    = params[:res]
        @arg    = params[:arg]
        @part   = params[:part]

        # set schema
        super(params)

    end

    def infer_cols
        self.cols = []
        self.cols << @res
        self.cols << @part unless @part.nil?
    end

    def infer_const

        self.const = Hash.new

        unless @part.nil?
            if @input.const.has_key? @part
                self.const[@part] = @input.const[@part]
            end
        end

    end

    #
    # Propagate domain information from input, considering renaming.
    #
    def infer_dom

        self.dom = Hash.new

        domsize = nil

        if @part.nil?
            domsize = 1
        else
            # REALLY fuzzy.  What we compute here is rather the average
            # value that is going to be computed by the count operator.
            self.dom[@part] = @input.dom[@part]
            domsize = self.card / self.dom[@part].size
        end

        self.dom[@res] = Domain.new :size => domsize

    end

    def infer_path

        self.path = Hash.new

        unless @part.nil?
            unless @input.path[@part].nil?
                self.path[@part] = @input.path[@part]
            end
        end

    end

    def infer_hist
        
        #
        # Can we do anything useful about histograms at all?
        # To play safe, don't propagate anything.
        #
        self.hist = Hash.new

    end

    def infer_card

        if @part.nil?
            self.card = 1
        else
            self.card = @input.dom[@part].size
        end

    end

    def to_dot_worker

        # FIXME
        <<EOT
        #{ nodename } [shape=record,label="Avg (#{ nodename })"];
EOT

    end

    def children
        [ @input ]
    end

    def evaluate_worker

        res = Result.new self.cols

        inp = @input.evaluate
        col = inp.col(@arg)

        if @part.nil?
            sum = 0.0
            @input.each { |t| sum += t[col].to_f }
            res << [ sum / @input.size ]
        else
            sum   = Hash.new
            count = Hash.new

            partcol = inp.col(@part)

            inp.tuples.each do |tuple|

                if !sum.has_key? tuple[partcol]
                    sum[tuple[partcol]] = 0.0
                    count[tuple[partcol]] = 0.0
                end
                sum[tuple[partcol]]   += tuple[col].to_f
                count[tuple[partcol]] += 1

            end

            sum.each_pair do |key,value|
                res << [ value / count[key], key ]
            end
        end

        res

    end

end
