#
# VLDB submission 'Dependable Cardinality Forecasts for XQuery'
# (Jens Teubner, Torsten Grust, Sebastian Maneth, and Sherif Sakr)
#
# (c) 2008 Jens Teubner, IBM Research, teubner@us.ibm.com
#
require 'algebra'

class FnNumber < AlgebraOperator


    #
    # Invoke with a hash containing
    #
    # +input+::      input relation
    # +args+::       a list that consists of one column name (or constant)
    # +res+::        result column name
    #
    def initialize(params)

        raise "input missing" unless params[:input]
        raise "arguments missing" unless params[:args]
        raise "result column name missing" unless params[:res]

        @input  = params[:input]
        @args   = params[:args]
        @res    = params[:res]

        # set schema
        super(params)

    end

    def infer_cols

        self.cols = Array.new @input.cols
        self.cols << @res

    end

    def infer_const

        self.const = @input.const.copy

        if @input.const[@args.first]
            self.const[@res] = @input.const[@args.first]
        end

    end

    def infer_dom
        
        self.dom = @input.dom.copy

        self.dom[@res] = @input.dom[@args.first]
        
    end

    def infer_path
        
        self.path = @input.path

    end

    def infer_hist
        
        self.hist = @input.hist.copy

        if @input.hist[@args.first]
            self.hist[@res] = @input.hist[@args.first]
        end

    end

    #
    # Binary Operators do not affect cardinality
    #
    def infer_card

        self.card = @input.card

    end

    def to_dot_worker

        ret = <<EOT
        #{ nodename } [shape=record,label="{FnNumber (#{ nodename }) | #{
            @res.to_s + ":(" + @args.to_s + ")"
        } }"];
EOT

        ret
    end

    def children
        [ @input ]
    end

    def evaluate_worker

        # new attribute is last
        res = Result.new self.cols

        inp = @input.evaluate

        argcol = inp.col(@args.first)

        inp.tuples.each do |tuple|

            new_tuple = Array.new tuple
            new_tuple << tuple[argcol]
            res << new_tuple

        end

        res

    end

end
