#
# VLDB submission 'Dependable Cardinality Forecasts for XQuery'
# (Jens Teubner, Torsten Grust, Sebastian Maneth, and Sherif Sakr)
#
# (c) 2008 Jens Teubner, IBM Research, teubner@us.ibm.com
#
require 'algebra'

# Generic class.  You'll likely want to use one of the subclasses instead.
class Not < AlgebraOperator

    #
    # Invoke with a hash containing
    #
    # +input+::      input relation
    # +arg+::        argument column name
    # +newcol+::     result column name
    #
    def initialize(params)

        raise "input missing" unless params[:input]
        raise "argument column name missing" unless params[:arg]
        raise "result column name missing" unless params[:newcol]

        @input  = params[:input]
        @arg    = params[:arg]
        @newcol = params[:newcol]

        # set schema
        super(params)

    end

    def infer_cols

        self.cols = Array.new @input.cols
        self.cols << @newcol

    end

    def infer_const

        self.const = @input.const.copy

        # see if argument 1 is actually a constant
        if @input.const[@arg]
            self.const[@newcol] = !@input.const[@arg]
        elsif @arg.class != Symbol
            self.const[@newcol] = !@arg
        end

    end

    def infer_dom
        
        self.dom = @input.dom.copy

        self.dom[@newcol] = Domain.new :size => @input.dom[@arg].size

    end

    def infer_path
        
        self.path = @input.path

    end

    def infer_hist
        
        self.hist = @input.hist.copy

        hist = nil

        if @input.hist[@arg1]
            hist = @input.hist[@arg1]
            self.hist[@newcol] = Histogram.new(hist.values.map { |v| !v })
        end

    end

    #
    # Binary Operators do not affect cardinality
    #
    def infer_card

        self.card = @input.card

    end

    def to_dot_worker

        ret = <<EOT
        #{ nodename } [shape=record,label="{Not (#{ nodename }) | #{
            @newcol.to_s + ":(" + @arg.to_s + ")"
        } }"];
EOT

        ret
    end

    def children
        [ @input ]
    end

    def evaluate_worker

        # new attribute is last
        res = Result.new self.cols

        inp = @input.evaluate

        argcol = inp.col(@arg)

        raise "problem finding column #{ @arg } from op #{ @input.nodename }" if argcol.nil?

        inp.tuples.each do |tuple|

            new_tuple = Array.new tuple
            new_tuple << !tuple[argcol]

            res << new_tuple

        end

        res

    end

end
