#!/usr/bin/ruby
#
# VLDB submission 'Dependable Cardinality Forecasts for XQuery'
# (Jens Teubner, Torsten Grust, Sebastian Maneth, and Sherif Sakr)
#
# (c) 2008 Jens Teubner, IBM Research, teubner@us.ibm.com
#

require 'getoptlong'

require 'algebra'

opts = GetoptLong.new(
    ['--dot', '-d', GetoptLong::NO_ARGUMENT],
    ['--properties', '-p', GetoptLong::NO_ARGUMENT],
    ['--evaluate', '-e', GetoptLong::NO_ARGUMENT],
    ['--compare', '-c', GetoptLong::NO_ARGUMENT],
    ['--serialize', '-s', GetoptLong::NO_ARGUMENT],
    ['--dump-xml-store', '-D', GetoptLong::NO_ARGUMENT]
)

dot = false
properties = false
evaluate = false
compare = false
serialize = false
dump_xml_store = false

opts.each do |opt, arg|

    case opt
    when '--dot'
        dot = true

    when '--properties'
        properties = true

    when '--evaluate'
        evaluate = true

    when '--compare'
        compare = true

    when '--serialize'
        serialize = true

    when '--dump-xml-store'
        dump_xml_store = true

    end

end

if ARGV.length != 1
    puts "Missing .rb file that holds query plan"
    exit -1
end

planfile = ARGV.shift

load planfile

raise "Problem with query plan file: no `$root' variable defined." unless $root

if dump_xml_store
    $root.evaluate
    XMLStore::dump
end

if dot
    puts "digraph Plan {\n"
    puts "ordering=out;\n"
    puts "node [shape=rectangle];\n"
    puts $root.to_dot(Array.new)
    puts "}\n"
end

if properties
    $root.infer_all
    $root.dump_properties
end

if evaluate
    result = $root.evaluate
    puts result
end

if serialize
    result = $root.evaluate
    #puts result.class
    result.serialize
end

if compare
    #print "\\draw\n"
    print <<EOT
%
% Query #{ planfile }
%
% Operator type, node name, estimated cardinality, actual cardinality, log
%
EOT
    $root.compare
    #print "  ;\n"
end
