#
# VLDB submission 'Dependable Cardinality Forecasts for XQuery'
# (Jens Teubner, Torsten Grust, Sebastian Maneth, and Sherif Sakr)
#
# (c) 2008 Jens Teubner, IBM Research, teubner@us.ibm.com
#
require 'algebra'

class TextConstr < AlgebraOperator

    #
    # Invoke with a hash containing
    #
    # +input+::   right input relation (input expression)
    # +iter+::      column with `iter' role
    # +item+::      column with `item' role
    #
    # Both input relations need to have iter,pos,item schema!
    #
    def initialize(params)

        raise "No input relation given" unless params[:input]
        raise "No `iter' column name given" unless params[:iter]
        raise "No `item' column name given" unless params[:item]

        @input = params[:input]
        @iter = params[:iter]
        @item = params[:item]

        raise "Wrong number of parameters in argument #{ @input.nodename }" unless @input.cols.size == 2

        # set schema
        super(params)

    end

    def infer_cols

        self.cols = [@iter,:pos,@item]

    end

    def infer_const

        self.const = Hash.new

    end

    def infer_dom
        
        self.dom = Hash.new

        self.dom[@iter] = @input.dom[@iter]
        self.dom[:pos]  = Domain.new :size => 1
        self.dom[@item] = Domain.new :size => self.card

    end

    def infer_path
        
        self.path = Hash.new

    end

    def infer_hist
        
        self.hist = Hash.new

    end

    def infer_card

        self.card = @input.card

    end

    def to_dot_worker

        ret = <<EOT
        #{ nodename } [shape=record,label="TextConstr (#{ nodename })"];
EOT

        ret
    end

    def children
        [ @input ]
    end

    #
    # Note: Does not (yet?) consider order in its input (i.e., the pos column)
    #
    def evaluate_worker

        res = Result.new self.cols

        input = @input.evaluate

        citer = input.col(@iter)
        citem = input.col(@item)

        input.tuples.each do |tuple|

            res << [ tuple[citer], 1, XMLStore::new_text(tuple[citem].to_s) ]

        end

        res

    end

end
