#
# VLDB submission 'Dependable Cardinality Forecasts for XQuery'
# (Jens Teubner, Torsten Grust, Sebastian Maneth, and Sherif Sakr)
#
# (c) 2008 Jens Teubner, IBM Research, teubner@us.ibm.com
#
require 'algebra'

# Generic class.  You'll likely want to use one of the subclasses instead.
class FunOneToOneImpl < AlgebraOperator

    #
    # Invoke with a hash containing
    #
    # +input+::      input relation
    # +arg1+::       first argument column name
    # +arg2+::       second argument column name
    # +newcol+::     result column name
    #
    def initialize(params)

        raise "input missing" unless params[:input]
        raise "argument list missing" unless params[:args]
        raise "result column name missing" unless params[:res]

        @input  = params[:input]
        @args   = params[:args]
        @res    = params[:res]

        raise "argument list must be an array" unless @args.instance_of? Array

        # set schema
        super(params)

    end

    def infer_cols

        self.cols = Array.new @input.cols
        self.cols << @newcol

    end

    def infer_const

        self.const = @input.const.copy

        const_args = []

        @args.each do |arg|

            if @input.const[arg]
                const_args << @input.const[arg]
            elsif arg.class != Symbol
                const_args << arg
            end

        end

        if const_args.size == args.size
            self.const[@res] = op const_args
        end

    end

    def infer_dom
        
        self.dom = @input.dom.copy

        self.dom[@res] = Domain.new :size => maxdomsize

    end

    def maxdomsize
        @input.card
    end

    def infer_path
        self.path = @input.path
    end

    def infer_hist
        self.hist = @input.hist.copy
    end

    #
    # Binary Operators do not affect cardinality
    #
    def infer_card
        self.card = @input.card
    end

    def to_dot_worker

        ret = <<EOT
        #{ nodename } [shape=record,label="{OneToOneImpl (#{ nodename }) | #{
            @newcol.to_s + ":(" + @args.to_s + ")"
        } }"];
EOT

        ret
    end

    def children
        [ @input ]
    end

    def evaluate_worker

        # new attribute is last
        res = Result.new self.cols

        inp = @input.evaluate

        argcols = @args.map { |arg| inp.col(arg) }

        inp.tuples.each do |tuple|

            new_tuple = Array.new tuple

            new_tuple << op(argcols.map { |argcol| tuple[argcol] })

            res << new_tuple

        end

        res

    end

end
