#
# VLDB submission 'Dependable Cardinality Forecasts for XQuery'
# (Jens Teubner, Torsten Grust, Sebastian Maneth, and Sherif Sakr)
#
# (c) 2008 Jens Teubner, IBM Research, teubner@us.ibm.com
#
require 'algebra'

class Distinct < AlgebraOperator

    #
    # Invoke with a hash containing
    #
    # +input+:: the input relation
    #
    def initialize(params)

        raise "No input relation given" unless params[:input]

        @input = params[:input]

        # set schema
        super(params)

    end

    def infer_cols
        self.cols = @input.cols
    end

    def infer_const
        self.const = @input.const
    end

    #
    # Propagate domain information from input, considering renaming.
    #
    def infer_dom
        self.dom = @input.dom
    end

    def infer_path
        self.path = @input.path
    end

    def infer_hist
        
        #
        # Can we do anything useful about histograms at all?
        # To play safe, don't propagate anything.
        #
        self.hist = Hash.new

    end

    def infer_card

        if @input.cols.length == 1
            self.card = @input.dom[@input.cols.first].size
        else
            # Stupid heuristics: use input cardinality * 80% for
            # distinct operators over multiple columns
            self.card = 0.8 * @input.card
        end

    end

    def to_dot_worker

        <<EOT
        #{ nodename } [shape=record,label="Distinct (#{ nodename })"];
EOT

    end

    def children
        [ @input ]
    end

    def evaluate_worker

        res = Result.new self.cols

        inp = @input.evaluate
        uniq = inp.tuples.uniq

        uniq.each do |tuple|
            res << Array.new(tuple)
        end

        res

    end

end
