#
# VLDB submission 'Dependable Cardinality Forecasts for XQuery'
# (Jens Teubner, Torsten Grust, Sebastian Maneth, and Sherif Sakr)
#
# (c) 2008 Jens Teubner, IBM Research, teubner@us.ibm.com
#
require 'algebra'

# Currently only works properly with single-column tables.
class Intersect < AlgebraOperator

    #
    # Invoke with a hash containing
    #
    # +left_rel+::  left input relation
    # +right_rel+:: right input relation
    #
    def initialize(params)

        raise "No left input relation given" unless params[:left_rel]
        raise "No right input relation given" unless params[:right_rel]

        @left_rel = params[:left_rel]
        @right_rel = params[:right_rel]

#        raise "Schemata do not match: [ #{ @left_rel.cols } ] " +
#            "vs. [ #{ @right_rel.cols } ]" unless
#                                              @left_rel.cols == @right_rel.cols

        # set schema
        super(params)

    end

    def infer_cols

        # evaluate depends on this to be the left rel
        self.cols = @left_rel.cols

    end

    def infer_const

        self.const = Hash.new

        @left_rel.const.each do |key, value|
            self.const[key] = value if @right_rel.const[key] == value
        end

    end

    # Haven't actually verified this rule (neither its implementation,
    # nor the basic idea).
    def infer_dom
        
        self.dom = Hash.new

        unless self.cols.size == 1
            #
            # There shouldn't be a fundamental problem here.  It's
            # just that I didn't want to think about it.
            #
            raise "Don't know how to implement multi-column intersect."
        end

        col = self.cols[0]

        common = Domain.common_superdomain(@left_rel.dom[col],
                                           @right_rel.dom[col])

        if common.nil?
            #
            # Same here. Didn't want to think about this.
            #
            raise "Don't know how to handle this."
        else

            domsize = @left_rel.dom[col].size * @right_rel.dom[col].size /
                common.size

            dom = Domain.new :size => domsize, :parent => @left_rel.dom[col]

            dom.subdomain_of! @right_rel.dom[col]

            self.dom[col] = dom

        end

    end

    def infer_path
        
        self.path = Hash.new

        @left_rel.path.each do |col, path|

            if @right_rel.path.has_key? col
                rpath = @right_rel.path[col]
                if rpath == path
                    self.path[col] = path
                else
                    self.path[col] = "((#{ path }) intersect (#{ rpath }))"
                end
            end
        end

    end

    def infer_hist
        
        self.hist = Hash.new

        # FIXME: To be done!

    end

    def infer_card

        unless self.cols.size == 1
            #
            # There shouldn't be a fundamental problem here.  It's
            # just that I didn't want to think about it.
            #
            raise "Don't know how to implement multi-column intersect."
        end

        self.card = self.dom[self.cols[0]].size

    end

    def to_dot_worker

        <<EOT
        #{ nodename } [shape=record,label="Intersect (#{ nodename })"];
EOT

    end

    def children
        [ @left_rel, @right_rel ]
    end

    def evaluate_worker

        if self.cols.size > 1
            # If we want to handle differences for multi-column tables,
            # we must be careful with the order of attributes
            raise "Intersect not implemented for multi-column tables."
        end

        res = Result.new self.cols

        l = @left_rel.evaluate
        r = @right_rel.evaluate

        intersect = (l.tuples & r.tuples).uniq

        intersect.each do |tuple|
            res << Array.new(tuple)
        end

        res

    end

end
